
function function_trace_image(image_file,root_file)

% function_trace_image : function to trace image
% Input:
% image_file: a tiff image file name.
% The green channel has the segments.
% The red channel has the juctions.
% root_file: a text file containing the pixel coordinates of the
% roots, which is the first point from where the tracing started.
% Output:  ".swc" files save under the swc/ folder.
% To view this swc file kindly open the tool Neuromantic under the tools/
% folder (only works with windows xp 32 bit system).
% 
% Code written by Jaydeep De.
% If you use this code kindly cite our work
% @Article{DeLiChe:Bmcbioinfo14,
%  author =       "Jaydeep De and Huiqi Li and Li Cheng",
%  title =        {Tracing retinal vessel trees by transductive inference},
%  journal =      "BMC Bioinformatics",
%  volume =       {15},
%  number =       {20},
%  pages =        {1-20},
%  YEAR =         {2014},
%  bib2html_pubtype = {Journal},
%  bib2html_dl_pdf = {http://www.biomedcentral.com/content/pdf/1471-2105-15-20.pdf}
% }

img=imread(image_file);
roots=load(root_file);
vessels=img(:,:,2);
branches=img(:,:,1);
figure(1),subplot(1,2,1),imshow(img),title(['Test Image:' image_file]);
%===============================================================
[terminals,leaf_point]=function_find_leafpoint(vessels,branches,roots);
if size(leaf_point,1)~=(size(terminals,1)-size(roots,1))
    fprintf('\n Error in finding leafs..exiting...');
    return;
end
%Finding the leaf points
%=================================================================

[Node,Connections,nodesegment,root_nodes]=function_create_node(vessels,branches,roots);
%create node structure
%=================================================================
[conn_num num1]=bwlabel(Connections);
conn_nodes=function_conn_nodes(conn_num,nodesegment);
%Finding the nodes connected to each junction
%=================================================================

%Finding the connecting points of each junction and their center

node_diadem=function_node_diadem(conn_num,nodesegment);
%=================================================================

node_ours=function_node_ours(nodesegment,node_diadem);
%Finding the Junctions for each segments (either one or two)
%=================================================================


%=================================================================
%Intiate ranking for first round



for i=1:size(leaf_point,1)
    if nodesegment(leaf_point(i,1),leaf_point(i,2))==0
        fprintf('\n Error in finding leafs..exiting...');
        return;
    end
end

rank_nodes=zeros(1,length(node_ours));

for i=1:size(leaf_point)
    rank_nodes(nodesegment(leaf_point(i,1),leaf_point(i,2)))=1;
end
%Intiate the ranking with leaf nodes
%=================================================================

[rank_nodes,critical_nodes]=function_rank_nodes_all(rank_nodes,Node,conn_nodes,nodesegment);

% return
% Correcting the vessels based on critical nodes
[vessels, branches]=function_correct_spur(Connections,nodesegment,rank_nodes,conn_nodes,critical_nodes);

%=====================================================================

%=====================================================================
% Ranking done for second time for the change of segments
[Node,Connections,nodesegment,root_nodes]=function_create_node(vessels,branches,roots);
%create node structure
%=================================================================
[conn_num num1]=bwlabel(Connections);
conn_nodes=function_conn_nodes(conn_num,nodesegment);
%Finding the nodes connected to each junction
%=================================================================

%Finding the connecting points of each junction and their center

node_diadem=function_node_diadem(conn_num,nodesegment);
%=================================================================

node_ours=function_node_ours(nodesegment,node_diadem);
%Finding the Junctions for each segments (either one or two)
%=================================================================

[terminals,leaf_point]=function_find_leafpoint(vessels,branches,roots);
if size(leaf_point,1)~=(size(terminals,1)-size(roots,1))
    fprintf('\n Error in finding leafs..exiting...');
    return;
end
%Finding the leaf points

%Intiate ranking for second round

for i=1:size(leaf_point,1)
    if nodesegment(leaf_point(i,1),leaf_point(i,2))==0
        fprintf('\n Error in finding leafs..exiting...');
        return;
    end
end

rank_nodes=zeros(1,length(node_ours));

for i=1:size(leaf_point)
    rank_nodes(nodesegment(leaf_point(i,1),leaf_point(i,2)))=1;
end
%Intiate the ranking with leaf nodes
%=================================================================

[rank_nodes,critical_nodes]=function_rank_nodes_all(rank_nodes,Node,conn_nodes,nodesegment);

%==================================================================

%label_all_node=function_generate_gt(t_num,gt_cell,roots,nodesegment);


numroots=size(roots,1);

[W,Y]=function_W_Y(Node,Connections,nodesegment,rank_nodes,numroots);
W_cap=W/max(max(W));
W_cap_1=W_cap';

D_dia=sum(W,1);
P=W;
for i=1:length(Node)
    if D_dia(i)~=0
        P(:,i)=P(:,i)/D_dia(i);
    end
end
W_cap_2=P';

%tau= [ 0.00001 0.0001 0.001 0.01 0.1 1 10 100 1000 10000];
%for kk=1:length(tau)
tau=10^6;
[F_P,MF]=function_Mf1(W_cap_1,Y,tau);
[maxval,label]=max(F_P,[],2);
[label,temp_image]=function_assing_label(F_P,nodesegment,roots);
figure(1),subplot(1,2,2),imagesc(temp_image),title('Tracing result');


for ii=1:max(max(temp_image))
    new_image=temp_image==ii;
    new_image(branches==1)=1;
    
    [conn_comp,num]=bwlabel(new_image);
    conn_comp_num=[];
    for i=1:num
        conn_comp_num(i)=length(find(conn_comp==i));
    end
    [dummy,idx]=max(conn_comp_num);
    
    vessels=(conn_comp==idx) & (temp_image==ii);
    Connections=(conn_comp==idx) & branches;
    Nodes=vessels;
    [nodesegment num]=bwlabel(Nodes);
    %Connections=branches;
    [conn_num num1]=bwlabel(Connections);
    
    [Node,Connections,nodesegment,root_nodes]=function_create_node(vessels,Connections,roots);
    
    %create node structure
    %=================================================================
    
    [conn_nodes,conn_num]=function_conn_nodes(conn_num,nodesegment);
    %Finding the nodes connected to each junction
    %if the number of nodes connected to a junction in 1 then remove that node.
    new_conn_nodes=cell(0,0);
    c=0;
    for i=1:length(conn_nodes)
        if length(conn_nodes{i}.ids)~=1
            c=c+1;
            new_conn_nodes{c}=conn_nodes{i};
        else
            Connections(conn_num==i)=0;
        end
    end
    [conn_num num1]=bwlabel(Connections);
    conn_nodes=new_conn_nodes;
    %=================================================================
    
    %Finding the connecting points of each junction and their center
    
    node_diadem=function_node_diadem(conn_num,nodesegment);
    %=================================================================
    
    [new_conn_nodes,new_node_diadem]=function_find_new_conn_nodes(...
        Node,conn_nodes,node_diadem,nodesegment);
    
    node_ours=function_node_ours(nodesegment,new_node_diadem);
    %Finding the Junctions for each segments (either one or two)
    %=================================================================
    
    
    
    adj=function_find_adj(nodesegment,new_conn_nodes,node_ours);
    
    single_root=roots(ii,:);
    root_node_num=root_nodes(root_nodes~=0);
    if nodesegment(single_root(1),single_root(2))~=root_node_num
        fprintf('\n Errot on root finding..exit code');
        return;
    end
    
    [d dt  pred] = bfs(adj,root_node_num);
    [ignore order] = sort(dt);
    if i==1
        c=0;
    end
    c=0;
    % fprintf('\nDoing tree %d\n',i);
    [swc_mat,c]=function_swc_from_gt_for_trace(nodesegment,conn_num,new_node_diadem,new_conn_nodes,order,pred,dt,single_root,c+1);
    %final_swc=[final_swc;swc_mat];
    %function_save_swc(['test/1_' num2str(i) '.swc'],swc_mat) ;
    function_save_swc(['swc/' image_file(7:end-4) '_' num2str(ii)  '_trace.swc'],swc_mat) ;
end
end