function function_trace_image_to_swc_file(image_file,root_pixel)
% function_trace_image_to_swc_file : function to create swc file from two
% channel image file.
% To know about the ".swc" file format kindly read the read me file.
% Input:
% image_file: a tiff image file name.
% The green channel has the segments.
% The red channel has the juctions.
% root_pixel: a text file containing the pixel coordinates of the
% root, which is the first point from where the tracing started.
% Output: A ".swc" file save under the current folder.
% To view this swc file kindly open the tool Neuromantic under the cuttent
% folder (only works with windows xp 32 bit system).
% 
% Code written by Jaydeep De.
% If you use this code kindly cite our work
% @Article{DeLiChe:Bmcbioinfo14,
%  author =       "Jaydeep De and Huiqi Li and Li Cheng",
%  title =        {Tracing retinal vessel trees by transductive inference},
%  journal =      "BMC Bioinformatics",
%  volume =       {15},
%  number =       {20},
%  pages =        {1-20},
%  YEAR =         {2014},
%  bib2html_pubtype = {Journal},
%  bib2html_dl_pdf = {http://www.biomedcentral.com/content/pdf/1471-2105-15-20.pdf}
% }

fprintf('\n Reading the image:%s\n',image_file);
figure,imshow(imread(image_file)),title('Green: Segments, Red: Junctions')
image=imread(image_file);
vessels=image(:,:,2);
Connections=image(:,:,1);

roots=load(root_pixel);
Nodes=vessels;

[conn_num num1]=bwlabel(Connections);

[Node,Connections,nodesegment]=function_create_node(vessels,Connections,roots);

%create node structure
%=================================================================

[conn_nodes,conn_num]=function_conn_nodes(conn_num,nodesegment);
%Finding the nodes connected to each junction
%if the number of nodes connected to a junction in 1 then remove that node.
new_conn_nodes=cell(0,0);
c=0;
for i=1:length(conn_nodes)
   if length(conn_nodes{i}.ids)~=1
       c=c+1;
       new_conn_nodes{c}=conn_nodes{i};
   else
       Connections(conn_num==i)=0;
   end
end
[conn_num num1]=bwlabel(Connections);
conn_nodes=new_conn_nodes;
%=================================================================

%Finding the connecting points of each junction and their center

node_diadem=function_node_diadem(conn_num,nodesegment);
%=================================================================

[new_conn_nodes,new_node_diadem]=function_find_new_conn_nodes(...
    Node,conn_nodes,node_diadem,nodesegment);

node_ours=function_node_ours(nodesegment,new_node_diadem);
%Finding the Junctions for each segments (either one or two)
%=================================================================



adj=function_find_adj(nodesegment,new_conn_nodes,node_ours);

single_root=roots;
root_node_num=nodesegment(single_root(1),single_root(2));

[d dt  pred] = bfs(adj,root_node_num);
[ignore order] = sort(dt);
if i==1
    c=0;
end
c=0;
% fprintf('\nDoing tree %d\n',i);
[swc_mat,c]=function_swc_from_gt_for_trace(nodesegment,conn_num,new_node_diadem,new_conn_nodes,order,pred,dt,single_root,c+1);
%final_swc=[final_swc;swc_mat];
%function_save_swc(['test/1_' num2str(i) '.swc'],swc_mat) ;
fprintf('Saving the .swc file: swc_file.swc\n');
function_save_swc(['swc_file.swc'],swc_mat) ;